module StrawberryAPI
  class Client < StrawberryAPI::HttpClient
    module Projects

      # Fetches all projects
      #
      # @option [Boolean] include_libraries false Include project libraries
      #
      # @return [Array<StrawberryAPI::Project>] A list of projects
      #
      def projects(include_libraries: false)
        projects = get("/projects").parse['projects']&.map do |project|
          Project.new(project)
        end

        if include_libraries
          projects
        else
          projects.keep_if do |project|
            !project.is_library_project
          end
        end
      end

      # Fetches all library projects
      #
      #
      # @return [Array<StrawberryAPI::Project>] A list of library projects
      #
      def library_projects
        projects = get("/projects").parse['projects']&.map do |project|
          Project.new(project)
        end

        projects.keep_if do |project|
          project.is_library_project
        end
      end

      # Fetches all online projects
      #
      #
      # @return [Array<StrawberryAPI::Project>] A list of online projects
      #
      def online_projects
        projects.select do |project|
          project.archive_strategy_id.nil? && !project.deleted && !project.is_library_project
        end
      end

      # Fetches all archived projects
      #
      #
      # @return [Array<StrawberryAPI::Project>] A list of archived projects
      #
      def archived_projects
        projects.select do |project|
          project.archive_strategy_id && !project.deleted && !project.is_library_project
        end
      end

      # Fetches a project
      #
      # @param [Integer] id Id of the project to retrieve
      #
      # @return [StrawberryAPI::Project] The fetched project
      #
      def project(id:)
        data = get("/projects/#{id}").parse['project']
        data.nil? ? nil : Project.new(data)
      end

      # Creates a project
      #
      # @param [String] name Name of the project to create
      # @param [String] templatename Name of the template to use
      #
      # @return [StrawberryAPI::Project] The created project
      #
      def create_project(name:, templatename:, custom_metadata: nil)
        if custom_metadata
            custom_metadata = custom_metadata.each_with_object(Array.new) do |field, memo|
            memo.push({custom_metadata_field: field[0], value: field[1]})
          end
        end

        body = {
          name: name,
          templatename: templatename,
          custom_metadata: custom_metadata
        }.to_json

        job = post("/projects", body: body).parse['job']

        project(id: job['project_id'])
      end

      # Updates a project
      #
      # @param [Interger] id Id of the project to update
      # @param [Hash] options Hash of options
      #
      # @return [StrawberryAPI::Project] The updated project
      #
      def update_project(id:, options: {})
        data = put("/projects/#{id}", body: options.to_json).parse['project']
        data.nil? ? nil : Project.new(data)
      end

      # Deletes a project
      #
      # @param [Integer] id Id of the project to delete
      #
      # @return [Boolean] Success
      #
      def delete_project(id:)
        delete("/projects/#{id}").success?
      end

      # Opens a project
      #
      # @param [Integer] id Id of the project to open
      #
      # @return [Boolean] Success
      #
      def open_project(id:)
        put("/projects/#{id}/open").success?
      end

      # Syncs a project
      #
      # @param [Integer] id Id of the project to sync
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def sync_project(id:)
        data = put("/projects/#{id}/sync").parse['job']
        data.nil? ? nil : ProjectFeedback.new(data)
      end

      # Freezes a project
      #
      # @param [Integer] id Id of the project to freeze
      #
      # @return [Boolean] Success
      #
      def freeze_project(id:)
        put("/projects/#{id}/freeze").success?
      end

      # Unfreezes a project
      #
      # @param [Integer] id Id of the project to unfreeze
      #
      # @return [Boolean] Success
      #
      def unfreeze_project(id:)
        delete("/projects/#{id}/unfreeze").success?
      end

      # Closes a project
      #
      # @param [Integer] id Id of the project to close
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def close_project(id:, edit:)
        delete("/projects/#{id}/close", headers: {'X-FLAVOURSYS-EDIT' => edit}).parse['job']
        data.nil? ? nil : ProjectFeedback.new(data)
      end

      # Forcecloses a project
      #
      # @param [Integer] id Id of the project to forceclose
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def forceclose_project(id:, edit:)
        data = delete("/projects/#{id}/forceclose", headers: {'X-FLAVOURSYS-EDIT' => edit}).parse['job']
        data.nil? ? nil : ProjectFeedback.new(data)
      end

      # Mounts a project
      #
      # @param [Integer] id Id of the project to mount
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def mount_project(id:, edit:)
        data = put("/projects/#{id}/mount", headers: {'X-FLAVOURSYS-EDIT' => edit}).parse['job']
        data.nil? ? nil : ProjectFeedback.new(data)
      end

      # Unmounts a project
      #
      # @param [Integer] id Id of the project to unmount
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def umount_project(id:, edit:)
        data = delete("/projects/#{id}/umount", headers: {'X-FLAVOURSYS-EDIT' => edit}).parse['job']
        data.nil? ? nil : ProjectFeedback.new(data)
      end

      # Refreshes a mounted project
      #
      # @param [Integer] id Id of the mounted project to refresh
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def refresh_mounted_project(id:)
        data = put("/projects/#{id}/refreshmountedproject", headers: {'X-FLAVOURSYS-EDIT' => edit}).parse['job']
        data.nil? ? nil : ProjectFeedback.new(data)
      end

      # Archives a project
      #
      # @param [Integer] id Id of the project to archive
      #
      # @return [StrawberryAPI::ProjectFeedback] The operation status
      #
      def archive_project(id:, archive_strategy_id:, exclude_linked_files: 'false')
        body = {
          strategy: archive_strategy_id,
          exclude_linked_files: exclude_linked_files
        }.to_json

        data = put("/projects/#{id}/archive", body: body).parse['archivestrategystate']
        data.nil? ? nil : ArchiveStrategyState.new(data)
      end

      # Unarchives a project
      #
      # @param [Integer] id Id of the project to unarchive
      #
      # @return [StrawberryAPI::ArchiveStrategyState] The operation status
      #
      def unarchive_project(id:)
        data = put("/projects/#{id}/unarchive").parse['archivestrategystate']
        data.nil? ? nil : ArchiveStrategyState.new(data)
      end

      # Searches for a substring in project names
      #
      # @param [String] name Substring to look for
      #
      # @return [Array<StrawberryAPI::Project>] A list of project matching the substring
      #
      def search_project(name:)
        get("/projects/search", body: {name: name}).parse['projects']&.map do |project|
          Project.new(project)
        end
      end

      # Fetches all teams assigned to a project
      #
      # @param [Interger] id If of the project to retrieve teams from
      #
      # @return [Array<StrawberryAPI::Team>] The fetched project teams
      #
      def project_teams(id:)
        teams = get("/projects/#{id}/teams").parse['array']
         teams.map do |team|
          Team.new(team.first)
        end
      end

      # Fetches all access rights of a project
      #
      # @param [Integer] id Id of the project to retrieve access rights from
      #
      # @return [Array<StrawberryAPI::AccessRight>] The fetched project access rights
      #
      def project_effective_access_rights(id:)
        access_rights = get("/projects/#{id}/effective_access_rights").parse['hash']
        access_rights.map do |accessright|
          AccessRight.new(accessright)
        end
      end

      # Assigns a team to a project
      #
      # @param [Integer] id Id of the project to which the team should be added
      # @param [Integer] team_id Id of the team to add to the project
      # @param [Boolean] write false Write access to the project
      #
      # @return [Boolean] Success
      #
      def add_team_to_project(id:, team_id:, write: false)
        body = {
          team_id: team_id,
          write: write
        }.to_json

        put("/projects/#{id}/teams", body: body).success?
      end

      # Removes a team from a project
      #
      # @param [Integer] id Id of the project from which the team should be deleted
      # @param [Integer] team_id Id of the team to remove from the project
      #
      # @return [Boolean] Success
      #
      def remove_team_from_project(id:, team_id:)
        delete("/projects/#{id}/teams/#{team_id}").success?
      end

      # Fetches a project status flags
      #
      # @param [Integer] id Id of the project to retrieve status flags from
      #
      # @return [Hash] The fetched project status flags
      #
      def project_status_flags(id:)
        status_flags = get("/projects/#{id}/status_flags").parse['array']
      end

      # Fetches a project custom metadata
      #
      # @param [Integer] id Id of the project to retrieve custom metadata from
      #
      # @return [Array<StrawberryAPI::CustomMetadata>] The fetched project custom metadata
      #
      def project_custom_metadata(id:)
        custom_metadata = get("/projects/#{id}/custom_metadata").parse['array']
        custom_metadata.map do |custom_metadata|
          CustomMetadatum.new(custom_metadata)
        end
      end

      # Updates an project custom metadata
      #
      # @param [Integer] id Id of the project which to update custom metadata
      # @param [Hash] custom_metadata Custom metadata to update the project with
      #
      # @return [Boolean] Success
      #
      def update_project_custom_metadata(id:, custom_metadata: nil)
        filthy_hash = StrawberryAPI::CustomMetadatum.filthify(custom_metadata: custom_metadata)
        put("/projects/#{id}/update_metadata", query: filthy_hash).success?
      end

      # Fetches a project assets
      #
      # @param [Integer] id Id of the project to retrieve which to retrieve assets from
      #
      # @return [Array<StrawberryAPI::Asset>] The fetched project assets
      #
      def project_assets(id:)
        assets = get("/projects/#{id}/assets").parse['assets']
        assets.map do |asset|
          Asset.new(asset)
        end
      end

      # Fetches projects size
      #
      #
      # @return [Hash] The fetched projects size
      #
      def projects_size
        get("/total_project_size").parse
      end

      # Fetches projects report
      #
      #
      # @return [Hash] The fetched projects report
      #
      def projects_repor
        get("/projects_report").parse
      end

    end
  end
end
